#ifndef		KERNEL_PLUGIN_H
#define		KERNEL_PLUGIN_H

#include		<windows.h>

#define		INITIAL_MICROKERNEL_MAJOR_VERSION			0
#define		INITIAL_MICROKERNEL_MINOR_VERSION			0
#define		INITIAL_MICROKERNEL_PATCH_NUMBER			0
#define		INITIAL_MICROKERNEL_BUILD_NUMBER			110

// This is used in many places
#ifndef HKEY_SQUEAK_ROOT
/* the default place in the registry to look for */
#define HKEY_SQUEAK_ROOT "SOFTWARE\\Squeak"
#endif

typedef int (*messageHook)(void *, unsigned int, unsigned int, long);

#define IMAGE_NAME_SIZE MAX_PATH

const char* KernelPlugin_getModuleName(void);
int KernelPlugin_initialiseModule(void);
int KernelPlugin_startupModule(void);
int KernelPlugin_shutdownModule(void);


typedef struct ModuleEntry 
{
	struct ModuleEntry *next;
	void								*handle;
	struct GenericProxy	*proxy;
	int									ffiLoaded;
	char								name[1];
} ModuleEntry ;

typedef struct GenericPlugin
{
//	Critical
	
	int			sig1 ;
	int			sig2 ;

// Compulsary
	struct	KernelProxy		*kernelProxy ;

	char*		(*getModuleName)(void) ;
	int			(*initialiseModule)(void) ;
	int			(*startupModule)(void) ;
	int			(*shutdownModule)(void) ;
	char**	(*getRequiredPlugins)(void) ;

// Optional	- MicroKernel will set defaults

	int			(*majorVersion)(void) ;	
	int			(*minorVersion)(void) ;	
	int			(*patchNumber)(void) ; 
	int			(*buildNumber)(void) ; 
	
	char*		(*getBuildDate)(void) ;
	char*		(*getBuildTime)(void) ;
	char*		(*getKnownProblems)(void) ;
	char*		(*getHistory)(void) ;
	char*		(*getCredits)(void) ;
	char*		(*getMaintainers)(void) ;
} GenericPlugin ;


typedef struct ProxySignature
{
	int		sig1 ;
	int		sig2 ;
} ProxySignature ;

//	Any proxy must be casted to GenericProxySignature to check for the signature 0xDEADCAFE
//  struct	GenericProxySignature		*genericProxySignature ;
//	genericProxySignature = (GenericProxySignature *)kernelProxy ;
// if(	genericProxySignature->proxySiganture->sig1 != 0xABCADACA ) ||
//			genericProxySignature->proxySiganture->sig2 != 0xDEADCAFE ) )
// then this is not a valid proxy !!!

typedef struct GenericProxySignature
{
	struct ProxySignature		*proxySignature ;
} GenericProxySignature ;

// Any proxy* can be casted to GenericProxy* to get at GenericPlugin stuff
typedef struct GenericProxy				
{
	struct GenericPlugin				*genericPlugin ;				
} GenericProxy				 ;

typedef struct WinMainParams
{
	HINSTANCE		hInst ;
	HINSTANCE		hPrevInst ;
  LPSTR				lpCmdLine ;
  int					nCmdShow	;
} WinMainParams	 ;

typedef struct KernelProxy 
{
	struct GenericPlugin		*genericPlugin ;				// Always first entry in a proxy
	struct WinMainParams		*winMainParams ;	

	struct GenericProxy*		(*kernelGetPluginProxy)(char *moduleName) ;
	HANDLE									(*ioDownLoadingPlugin)(char *pluginsDir, char *pluginName) ;
	BOOL										(*ioDownloadThisFtpOrHttpFileToThisLocalDirectory)(char *pszThisInternetFile, char *pszThisLocalDir, char *pszDstFilePathName) ;
	BOOL										(*ioGetUrlforThisPlugin)(char *pszIniFile, char *pszPluginsProfile, char *pszPluginName, char *pszUrl, int nSize) ;

	int											(*ioLoadFunctionFrom)(char *fnName, char *modName);
	ModuleEntry*						(*ioGetPluginModuleEntry)(char *pluginName) ;
	void*										(*ioGetPluginProxy)(char *pluginName) ;

	struct PlugManProxy			*plugManProxy ;
	struct VirtualMachine		*interpreterProxy ;

	TCHAR										*squeakIniName ;
	TCHAR										*pluginsProfile ;

	TCHAR										*vmPath ;
	TCHAR										*vmName ;

	char										*imageName ;
	TCHAR										*imagePath ;

} KernelProxy ;


struct KernelProxy* sqGetKernelProxy(void) ;

#endif		// KERNEL_PLUGIN_H
